#ifndef NETIO_H
#define NETIO_H

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include "types.h"

/*---	definitions taken from RFC 1350	---*/

#define PE_ERRMASK	0x100

#define	PE_UNDEF	0		/* not defined */
#define	PE_NOTFOUND	1		/* file not found */
#define	PE_ACCESS	2		/* access violation */
#define	PE_NOSPACE	3		/* disk full or allocation exceeded */
#define	PE_BADOP	4		/* illegal TFTP operation */
#define	PE_BADID	5		/* unknown transfer ID */
#define	PE_EXISTS	6		/* file already exists */
#define	PE_NOUSER	7		/* no such user */

#define	PX_UNDEF	(PE_UNDEF    | PE_ERRMASK)
#define	PX_NOTFOUND	(PE_NOTFOUND | PE_ERRMASK)
#define	PX_ACCESS	(PE_ACCESS   | PE_ERRMASK)
#define	PX_NOSPACE	(PE_NOSPACE  | PE_ERRMASK)
#define	PX_BADOP	(PE_BADOP    | PE_ERRMASK)
#define	PX_BADID	(PE_BADID    | PE_ERRMASK)
#define	PX_EXISTS	(PE_EXISTS   | PE_ERRMASK)
#define	PX_NOUSER	(PE_NOUSER   | PE_ERRMASK)

#define RT_RRQ		1		/* read request */
#define RT_WRQ		2		/* write request */
#define RT_DATA		3		/* data */
#define RT_ACK		4		/* acknowledgement */
#define RT_ERROR	5		/* error */

#define RMS_OCTET	"octet"
#define RMS_NETASCII	"netascii"
#define RMS_MAIL	"mail"		/* unsupported */

#define RM_OCTET	1
#define RM_NETASCII	2
#define RM_MAIL		3		/* unsupported */

#define SEGSIZE		512

typedef struct tftpH_s {	USHORT	opCode;
				union {
					USHORT blockno;
					SHORT errcode;
					CHAR  reqtext[1];
				 } th_u;			
					} tftpH_t;

typedef struct tftpD_s {	tftpH_t	head;
				CHAR	data[SEGSIZE];

					} tftpD_t;
#define	blockNo	th_u.blockno
#define	errCode	th_u.errcode
#define reqText	th_u.reqtext

/*---	end of RFC 1350 definitions	---*/

#define SEND_TIMEOUT	2
#define SACK_TIMEOUT	4
#define RECV_TIMEOUT	4
#define RACK_TIMEOUT	2

#define MAXRETRIES	5	/* timeout cycles for ACK receive */
#define MAXRETRANS	5	/* retransmissions per block */

typedef struct sockaddr    sa_t;
typedef struct sockaddr_in sai_t;

typedef struct tftpR_s {	INT	reqType;
				INT	reqMode;
				CHAR*	reqFile;
				FILE*	reqFd;
				INT	reqSock[2];
				sai_t	reqPeer;
				count_t	xfrstat;
				count_t	iostat;
				CHAR	*filter;
				CHAR	*realFile;

					} tftpR_t;

#define Rsock	reqSock[0]
#define Ssock	reqSock[1]

typedef struct sopair_s {	INT	recv;
				INT	send; } sopair_t;

CHAR *xinet_ntoa(in_addr_t ia);
INT isaSocketFd(INT fd);
INT isaSocket(FILE *fp);
INT sendToError(sopair_t *sockets, sai_t *peer, INT err, ...);
INT sendError(tftpR_t *request, INT err, ...);
sopair_t *netStart(VOID);
INT getRequest(sopair_t *sockets, tftpR_t *request, INT maxwait);
INT connectPeer(tftpR_t *request);
INT recvFile(tftpR_t *request);
INT sendFile(tftpR_t *request);

#endif /* NETIO_H */
